/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
dojo.provide("zero.widget.ValidatorForm");

dojo.require("dojo.widget.*");
dojo.require("zero.eventproxy");
dojo.require("dojo.string.extras");  



dojo.widget.defineWidget(
	// widget name and class
	"zero.widget.ValidatorForm",
	
	// superclass
	dojo.widget.Form,
	
	// properties and methods
	{
		id: "",
		name: "",
		EVENT_PROXY_URL: "/event", 
		FIELD_CHANGE_EVENT: "onFieldChanged",
		FORM_SUBMIT_EVENT: "onFormSubmitted",
		NO_ERROR_HIGHLIGHTING_COLOR: "greenyellow",
		ERROR_HIGHLIGHTING_COLOR: "salmon",
		isContainer:true,		
		templatePath: dojo.uri.dojoUri("zero", "widget/ValidatorForm.html"),
		templateCssPath: dojo.uri.moduleUri("zero", "widget/ValidatorForm.css"),
		submitButtonId: "",	
		submitButton: null,
		whoamiProp: "this is the ValidatorForm widget object (use this if you need to see if 'this' points to the right place)",
		errorDivs: new Object(),
		formDivs: new Object(),
		//METHODS:
		
		formSubmit: function(evt){
			if (!evt)
				evt=window.event;
			var returnCodeObj = {rc:true};			
			this.fireFormSubmitValidationEvent(this.containerNode, returnCodeObj);
			return returnCodeObj.rc;
		},
		
		handleFieldValidationResponse: function(type, data, evt, element){
			try{
				var massagedData = new Object();
				if (data){
					
					var errors = (data.getElementsByTagName("Error"));
					var error, fieldHighlighter, desiredHighlighting;
					var messages = new Array();
					if (errors && errors.length > 0){
					 	for (var i = 0 ; i < errors.length; i++){
					 		error = errors.item(i);
					 		messages.push(error.getAttribute("message"));					 		
						}
						desiredHighlighting = this.ERROR_HIGHLIGHTING_COLOR;
					} else {
						desiredHighlighting = this.NO_ERROR_HIGHLIGHTING_COLOR;
					}
					fieldHighlighter = this.fieldHighlighters[element.type];			 		
			 		if (fieldHighlighter)
			 			fieldHighlighter.call(this, element, desiredHighlighting, messages);			 		
				}
			} catch (e){
				alert("error in handler: " + e.message);
				dojo.debugDeep(e);
			}			
		  },
		  
		  handleFormValidationResponse: function(type, data, evt, form){
		  	try{
				var massagedData = new Object();
				if (data){					
					var errors = (data.getElementsByTagName("Error"));
					var error, fieldHighlighter, desiredHighlighting;
					var messages = new Array();
					if (errors && errors.length > 0){
						rc = false;
					 	for (var i = 0 ; i < errors.length; i++){
					 		error = errors.item(i);
					 		messages.push(error.getAttribute("message"));					 		
						}
						messages.push("This form was not submitted.  Please correct the errors listed above, and try again.");
						desiredHighlighting = this.ERROR_HIGHLIGHTING_COLOR;
					} else {
						desiredHighlighting = this.NO_ERROR_HIGHLIGHTING_COLOR;
					}
					fieldHighlighter = this.fieldHighlighters["form"];			 		
			 		if (fieldHighlighter)
			 			fieldHighlighter.call(this, form, desiredHighlighting, messages);			 		
				}
			} catch (e){
				alert("error in handler: " + e.message);
				dojo.debugDeep(e);
			}	
			return rc;
		  },
		  
		  postCreate: function(){		  		
		  		this.domNode.name = this.name;
		  		this.domNode.id = this.id;
		  		
		  		//this.findSubmitButton();
		  		this.setSubmitHandler();
		  		this.setOnChangeHandlers(); 	
		  	 		
		  },
		  
		
		  setOnChangeHandlers : function(){
		  	//dojo.debugDeep(this.formElementValueGetters);
		  	  for (var i = 0; i < this.containerNode.elements.length; i++){
					var valueGetter = this.formElementValueGetters[this.containerNode.elements[i].type];
					if (valueGetter){
						this.registerOnChangeHandler(this.containerNode.elements[i], valueGetter);
					} else {
						//alert("no valuegetter for type: " + this.containerNode.elements[i].type);
					}
		  	  }	  	  		  	  
		  },
		  
		  setSubmitHandler: function(){
		  	
	  			this.containerNode.onsubmit=dojo.lang.hitch(this, "formSubmit");		  		
		  },
		  
		  findSubmitButton: function(){
		  		//if (this.submitButtonId)
		  		//	this.submitButton = document.getElementById(this.submitButtonId);
		  				  		
		  		if (!this.submitButton){
		  			for (var i = 0; i < this.domNode.childNodes.length; i++){
		  				if ("submit" == this.domNode.childNodes[i].type){
		  					this.submitButton = this.domNode.childNodes[i];
		  					break;
		  				} else {
		  					//alert ('type was: ' + this.domNode.childNodes[i].type)
		  				}
		  			}
		  		}		  		
		  },
		  
		  getContentObjectForForm: function(form){
		  		var elem;
		  		var returnObj = new Object();
		  		returnObj.formName=form.name;
		  		for (var i = 0; i < form.elements.length; i++){
		  			elem = form.elements[i];
		  			if (returnObj[elem.name])
			  			continue;
		  			var valueGetterMethod = this.formElementValueGetters[elem.type];
		  			if (valueGetterMethod)
		  				returnObj[elem.name] = valueGetterMethod(elem);		  			
		  		}
		  		return returnObj;
		  },
		  
		  fireFormSubmitValidationEvent: function (form, returnObj){
		  	 var widget = this; //need to resolve "this" at time of fireFieldValidationEvent execution, not callback execution (this will point to diff. things)
		  	 
		  	 var callbackFunction = function(type, data, evt){
		  	 	 returnObj.rc = widget.handleFormValidationResponse(type, data, evt, form);
		  	 };
		  	
		  	 var contentObj = this.getContentObjectForForm(form);
		  	 var paramsObj = {    
			    content: contentObj,
			    mimetype: "application/xml",
			    load: callbackFunction,
			    sync: true
			 };
			 
			 var urlAppendage;
			 if (form && form.name)
			 	urlAppendage = form.name;		
			 zero.eventproxy.fireEvent(this.EVENT_PROXY_URL, this.FORM_SUBMIT_EVENT, paramsObj, urlAppendage);	    
		  },
		  
		  fireFieldValidationEvent: function (element, value){
		  	 var widget = this; //need to resolve "this" at time of fireFieldValidationEvent execution, not callback execution (this will point to diff. things)
		  	 var callbackFunction = function(type, data, evt){
		  	 	 widget.handleFieldValidationResponse(type, data, evt, element);
		  	 };
		  	
		  	 var paramsObj = {    
			    content: {fieldName: element.name, value: value},
			    mimetype: "application/xml",
			    load: callbackFunction
			 };
			 
			 var form = element.form;
			 var urlAppendage;
			 if (form && form.name)
			 	urlAppendage = form.name + "/" + element.name;		
			 zero.eventproxy.fireEvent(this.EVENT_PROXY_URL, this.FIELD_CHANGE_EVENT, paramsObj, urlAppendage);	    
		  },
		  
		  //element is the form element to have its handler set
		  registerOnChangeHandler : function(element, valueGetterMethod) {		  	
		  	//"this" refers to the form element being changed
		  	  var handlerToUse = this.handlerNames[element.type];
		  	  var formWidget = this;
		  	  element[handlerToUse] = function(evt){
		  	  	  if (!evt)
		  	  		  evt = window.event;
		  	  		  
		  	      //define this inner function
		  	      //inside this scope, "this" refers to the element
		  	      var elemThatChanged = this;		  	  	  	  
		  	      
		  	  	  var onChangeMethod = function(evt){
		  	  	  	  var value = valueGetterMethod(elemThatChanged);
		  	  	  	  formWidget.fireFieldValidationEvent(elemThatChanged, value);
		  	  	  };
		  	  	  
		  	  	  //invoke our inner function
		  	  	  onChangeMethod(evt);
		  	  };
		  },
		  
		  handlerNames : {
		  	  "radio" : "onclick", //may need to use onclick for this one
		  	  "text" : "onchange",
		  	  "password" : "onchange",
		  	  "textarea" : "onchange",
		  	  "select" : "onchange",
		  	  "hidden" : "onchange",
		  	  "checkbox" : "onclick"
		  },
		  		  		  
		  formElementValueGetters: {
		  	  radio : function (radioButton) {
		  	  	if (radioButton.checked)
			  		return radioButton.value;
			  	return null;
		  	  },
		  	  		  	 
		  	  text :  function (textField) {
		  	  		return textField.value;
		  	  },
		  	  
		  	  select :  function (selectField) {
		  	  		alert("getting value for select, it is: " + selectField.value);
		  	  		return selectField.value;
		  	  }, 
		  	  
		  	  textarea : function (textareaField) {
		  	  		alert("getting value for textarea, it is: " + textareaField.value);
		  	  		return textareaField.value;
		  	  },
		  	  
		  	  checkbox: function(checkboxField){
		  	  	  var formChild = checkboxField.form[checkboxField.name];
		  	  	  if (formChild.length != undefined){
		  	  	  	var values = new Array();
		  	  	  	for (var i = 0; i < formChild.length; i++){
		  	  	  		if (formChild[i].checked)
		  	  	  			values.push(formChild[i].value);
		  	  	  	}
		  	  	  	if (values.length > 0)
		  	  	  		return values.join(",");
		  	  	  } else {
		  	  	  	if (checkboxField.checked)
		  	  	  		return checkboxField.value;
		  	  	  }
		  	  	  return null;
		  	  }
		  	
		  },
		  
		  fieldHighlighters: {
		  	  text: function(element, color, messageArray){
		  	  		element.style.backgroundColor=color;		  	  		
		  	    	var newDiv = this.errorDivs[element.name];
		  	    	if (!newDiv){
		  	    		newDiv = document.createElement("div");
		  	    		element.parentNode.insertBefore(newDiv, element);
		  	    		this.errorDivs[element.name] = newDiv;
		  	    	}
					this.populateDivWithMessages(newDiv, messageArray);	  
		  	  }, 
		  	  
		  	  radio: function(element, color, messageArray){
		  	  		var radioArray = element.form[element.name];
		  	  		var elementToPutMessageBefore;
		  	  		
		  	  		if (radioArray.length){
			  	  		for (var i = 0; i < radioArray.length; i++)
			  	  			radioArray[i].style.backgroundColor=color;
			  	  		elementToPutMessageBefore = radioArray[0];
			  	  	} else {
			  	  		element.style.backgroundColor=color;
			  	  		elementToPutMessageBefore = element;
			  	    }
		  	  			
		  	  		var newDiv = this.errorDivs[element.name];
		  	    	if (!newDiv){
		  	    		newDiv = document.createElement("div");
		  	    		elementToPutMessageBefore.parentNode.insertBefore(newDiv, elementToPutMessageBefore);
		  	    		this.errorDivs[element.name] = newDiv;
		  	    	}	  	  			
		  	  		
					this.populateDivWithMessages(newDiv, messageArray);	  
		  	  },
		  	  
		  	  select: function(element, color, messageArray){
		  	  		element.style.backgroundColor=color;
		  	    	var newDiv = this.errorDivs[element];
		  	    	if (!newDiv){
		  	    		newDiv = document.createElement("div");
		  	    		element.parentNode.insertBefore(newDiv, element);
		  	    		this.errorDivs[element] = newDiv;
		  	    	}
					this.populateDivWithMessages(newDiv, messageArray);	  
		  	  }, 
		  	  textarea: function(element, color, messageArray){
		  	  		element.style.backgroundColor=color;
		  	    	var newDiv = this.errorDivs[element];
		  	    	if (!newDiv){
		  	    		newDiv = document.createElement("div");
		  	    		element.parentNode.insertBefore(newDiv, element);
		  	    		this.errorDivs[element] = newDiv;
		  	    	}
					this.populateDivWithMessages(newDiv, messageArray);	  
		  	  },
		  	  
		  	  checkbox: function(element, color, messageArray){
		  	  		var checkboxArray = element.form[element.name];
		  	  		
		  	  		//set all other members of this checkbox array to same color (the group is valid or invalid as a whole)
		  	  		if (checkboxArray.length){
			  	  		for (var i = 0; i < checkboxArray.length; i++)
			  	  			checkboxArray[i].style.backgroundColor=color;
			  	  		elementToPutMessageBefore = checkboxArray[0];
			  	  	} else {
			  	  		element.style.backgroundColor=color;
			  	  		elementToPutMessageBefore = element;
			  	    }
		  	  			
		  	  		var newDiv = this.errorDivs[element.name];
		  	    	if (!newDiv){
		  	    		newDiv = document.createElement("div");
		  	    		elementToPutMessageBefore.parentNode.insertBefore(newDiv, elementToPutMessageBefore);
		  	    		this.errorDivs[element.name] = newDiv;
		  	    	}
					this.populateDivWithMessages(newDiv, messageArray);	  
		  	  }, 
		  	  
		  	  form: function(form, color, messageArray){
		  	  	   var newDiv = this.formDivs[form.name];
		  	    	if (!newDiv){
		  	    		newDiv = document.createElement("div");
		  	    		form.parentNode.insertBefore(newDiv, form);
		  	    		this.formDivs[form.name] = newDiv;
		  	    	}			  	    	
		  	    	this.populateDivWithMessages(newDiv, messageArray);	  	 		  	  	
		  	  }
		  	
		  },
		  
		  populateDivWithMessages: function(div, messageArray){
		  	var newHtml = new Array();
  	    	for (var i = 0; i < messageArray.length; i++){
  	    		newHtml.push('<span style="color:red; font-face:Arial; font-size:10pt;">' + messageArray[i] + "</span>");  	    		
  	    	}
  	    	div.innerHTML = newHtml.join("<br>");
  	    	div.style.display = (div.innerHTML) ? "block" : "none"; //if it's empty, don't display it
		  }
		  
	}


);
testmethod = function(type, data, evt){
	try{
	
	alert(data.nodeType);
	alert('firstchild...');
	alert(data.firstChild.nodeName);
		alert(data.firstChild.nodeType);
			alert(data.firstChild.nodeValue);

	} catch (e){
		alert(e.message);
		//for (var prop in e){
		//	alert (prop + ": " + e[prop]);
		//}
		
	}
	for (var prop in data){
		alert(prop);
	}
}