<!--
/*
 * ============================================================================
 * (C) Copyright IBM Corporation 2006, 2007.  All Rights Reserved.
 * 
 * You may only copy, modify, and distribute these samples internally.
 * These samples have not been tested under all conditions and are provided
 * to you by IBM without obligation of support of any kind.
 *
 * IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
 * CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
 * CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
 * ============================================================================
 */
-->
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" 
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Using the RestStore with the Dojo Grid</title>

<style type="text/css">
	@import "dijit/themes/tundra/tundra.css";
    @import "dojox/grid/_grid/tundraGrid.css";
	@import "dojostyle.css";
</style>

<script type="text/javascript" 
    src="dojo/dojo.js"
    djConfig="parseOnLoad: true, isDebug: false"></script>

<script type="text/javascript">
    dojo.require("dojox.grid.Grid");
    dojo.require("zero.resource.RestStore");
    dojo.require("dijit.form.Form");
    dojo.require("dijit.form.ComboBox");
    dojo.require("dijit.form.TextBox");
    dojo.require("dojo.data.ItemFileReadStore");
    dojo.require("dojo.parser");

// get_relative_uri is used to make the link work independent of context roots, reverse proxies, etc.
var employeeURL = '<?php echo get_relative_uri("/resources/employees");?>';

function listEmployees(){
    setError("");
    // Using the jsId of the model to call refresh, which will reload the grid.
    employeeDataModel.refresh({onError: onError});
}

function addEmployee() {
	var empform = dijit.byId("empform");
	var values = empform.getValues();
	if (values.username == '') {
		setError("Username cannot be empty.");
		return;
	}

    var employeeItem = null;
    try {
        employeeItem = employeeStore.newItem(values);
        if (employeeItem) {
            onEmployeeAdded(employeeItem);
        }
    } catch (error) {
        setError(error);
    }
}

function deleteEmployee() {
	setError("");
    employeeDataGrid.removeSelectedRows();
}

function updateEmployee() {
	var editform = dijit.byId("editform");
  	var values = editform.getValues();
  	var username = dijit.byId("editUsername").getDisplayedValue();

    var itemFound = function(item, request) {
        if (item) {
           var key;
           for (key in values) {
               employeeStore.setValue(item, key, values[key]);
           }
           employeeStore.save({onComplete: onEmployeeUpdated, onError: onError});
        }
    };

    employeeStore.fetchItemByIdentity({identity: username, onItem: itemFound, onError: onError});
}


function onError(response) {
	setError(response);
}

function onEmployeeAdded(data) {		
	hideEmployeeForm(true);
	//listEmployees();
}

function onEmployeeUpdated(response) {
	//	we know we've been successful
	hideEditForm(true);
	//listEmployees();
}

function showEditForm() {
	hideEmployeeForm(false);
	
	var username = getSelectedUserName();
	if (username == null) {
		return;
	}
	
	var req = dojo.xhrGet({
		url: employeeURL + '/' + username,
		handleAs: 'json-comment-optional', 
		sync: false
	});
	req.addCallbacks(onShowEditForm, onError);
	var editformholder = dojo.byId("editformholder");
	editformholder.style.display = "block";
}
	
function onShowEditForm(data){
	var editform = dijit.byId("editform");
  	editform.setValues(data);
}

function showEmployeeForm() {
	hideEditForm(false);

	var empformholder = dojo.byId("empformholder");
	empformholder.style.display = "block";
}

function hideEmployeeForm(reset) {
	var empformholder = dojo.byId("empformholder");
	empformholder.style.display = "none";
	if (reset) {
		var empform = dijit.byId("empform");
		empform.containerNode.reset();
	}
	setError("");
}

function hideEditForm(reset) {
	var editformholder = dojo.byId("editformholder");
	editformholder.style.display = "none";
	if (reset) {
		var editform = dijit.byId("editform");
		editform.containerNode.reset();
	}
	setError("");
}

function getSelectedUserName() {
    var rowSelected = employeeDataGrid.selection.getSelected();
    var selectionData = employeeDataModel.data[rowSelected]

	if (selectionData != null) {
		return selectionData.username;
	}
}

function setError(errorText) {
	if (errorText != null && errorText != "") {
		errorText = '<font color="red">';
	    if (errorText.responseText && errorText.message !=null ) {
	   		errorText += errorText.message + "\n<br/>\n" +
	   			errorText.responseText.toString();
	    }else if(errorText.message != null ){
	    	errorText += errorText.message;
	    }else if(errorText.responseText != null ){
	    	errorText += errorText.responseText.toString();
	    }
	    errorText += "<br/>Check the error log for more details.</font>";
	}
	var err = dojo.byId("error");
	err.innerHTML = errorText;
}

<!-- display the employee info using the dojox Grid -->
var view = {cells:
            [[
			    {name: 'Username', field: 'username', width: 'auto'}, {name: 'First Name', field: 'firstname', width: 10}, {name: 'Last Name', field: 'lastname', width: 10}, {name: 'Location', field: 'location', width: 10}, {name: 'Phone Number', field: 'phonenumber', width: 10}
            ]]
           };
		// a grid layout is an array of views.
var layout = [ view ];
</script>

</head>

<body class="tundra">
    <script type="text/javascript">document.write('<div dojoType="zero.resource.RestStore" jsId="employeeStore" resourceUri="'+employeeURL+'" memberIdentifier="username" memberLabel="username"></div>');</script>
    <div dojoType="dojox.grid.data.DojoData" jsId="employeeDataModel" store="employeeStore" rowsPerPage="10"></div>
    <div style="width: 800px; margin: auto; border-width: 1px 1px 1px 1px; border-style: solid; border-color: lightgray;">
    <div jsid="employeeDataGrid" id="grid" dojoType="dojox.grid.Grid" model="employeeDataModel" structure="layout" style="width: 100%; height: 250px;"></div>
    <table style="width: 100%; border-width: 0px 0px 0px 0px; border: 0px; border-style: none;">
        <tbody>
            <tr>
                <td>
                    <button onclick="listEmployees();" style="width: 100%">List Employees</button>
                </td>
                <td>                    
                    <button onclick="showEmployeeForm();" style="width: 100%">Add New Employee</button>
                </td>
                <td>
                    <button onclick="showEditForm();" style="width: 100%">Edit Selected Employee</button>
                </td>
                <td>
                    <button onclick="deleteEmployee();" style="width: 100%">Delete Selected Employee</button>
                </td>
                <td>
                </td>
            </tr>
        </tbody>
    </table>
</div>
<div dojoType="dojo.data.ItemFileReadStore" jsId="locationStore" url="<?php echo get_relative_uri('/resources/locations');?>"></div>

<!-- form for creating an employee -->
<div id="empformholder" style="display:none">
<form dojoType="dijit.form.Form" id="empform" name="empform" onSubmit="return false;">
	<br/><br/>
	<table border=2> 
		<tr><th align="left">Property</th><th align="left">Value</th></tr> 
		<tr><td>Username</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="32" name="username"  /></td></tr> 
		<tr><td>First Name</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="16" name="firstname"  /></td></tr> 
		<tr><td>Last Name</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="16" name="lastname" /></td></tr>
		<tr><td>Location</td><td><select dojoType="dijit.form.ComboBox" store="locationStore" maxlength="64" name="location" /></td></tr>
		<tr><td>Phone Number</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="16" name="phonenumber" /></td></tr>
	</table><br />
    <button onclick="addEmployee();">Add</button>    
    <button onclick="hideEmployeeForm(true);">Cancel</button>
</form>
</div>

<!-- form for editing employee details -->
<div id="editformholder" style="display:none">
<form dojoType="dijit.form.Form" id="editform" name="editform" onSubmit="return false;">
<br/><br/>
<table border=2>
	<tr><th align="left">Property</th><th align="left">Value</th></tr> 
		<tr><td>Username</td><td><input type="text" id="editUsername" dojoType="dijit.form.TextBox" maxlength="32" name="username"  disabled/></td></tr> 
		<tr><td>First Name</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="16" name="firstname"  /></td></tr> 
		<tr><td>Last Name</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="16" name="lastname" /></td></tr>
		<tr><td>Location</td><td><select dojoType="dijit.form.ComboBox" store="locationStore" maxlength="64" id="edit_location" name="location"/></td></tr>
		<tr><td>Phone Number</td><td><input type="text" dojoType="dijit.form.TextBox" maxlength="16" name="phonenumber" /></td></tr>
	</table><br />
    <button onclick="updateEmployee();">Update</button>    
    <button onclick="hideEditForm(true);">Cancel</button>
</form>
</div>

<!-- display error message -->
<div id="error"></div>

</body>
</html>
